/*
 * connect the ESC's tlm to serial 1 RX pin and a fast diode from serial1 TX pin to serial1 RX pin (TX --|<-- RX)
 * on arduinos with more then one serials serial0 is used to display telemetry, and serial 1 for ESC communication. on others serial 0 is esc communication
 * 
 */


#if defined(__AVR_ATmega168__) || defined(__AVR_ATmega328P__) ||  defined(__AVR_ATmega8__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__) || defined(__AVR_ATmega32U4__) || defined(TEENSY20)
  #define AVR8
#endif
#if defined(__AVR_ATmega1280__) || defined(__AVR_ATmega1281__) || defined(__AVR_ATmega2560__) || defined(__AVR_ATmega2561__) || defined(__AVR_ATmega32U4__) || defined(TEENSY20)
  #define UART1_PRESENT
#endif

#ifdef AVR8
  uint8_t B2M_RXbuf[256] = {0};
  uint8_t B2M_RXhead = 0;
  uint8_t B2M_RXtail = 0;
#endif


void Arduino_USART_Init(){
  #ifdef UART1_PRESENT
   Serial.begin(115200);
  #endif
}

void B2M_USART_Init(){
  #ifndef AVR8
    Serial1.begin(2000000);
  #else
    #ifdef UART1_PRESENT
    B2M_USART1_Init();
    #else
    B2M_USART0_Init();
    #endif
  #endif
}

uint8_t B2M_USART_Byte_available(){
  #ifndef AVR8
    return Serial1.available();
  #else
    return (uint8_t)((uint8_t)B2M_RXhead - (uint8_t)B2M_RXtail);
  #endif
}

uint8_t B2M_USART_ReadByte(){
  #ifndef AVR8
    return Serial1.read();
  #else
    return B2M_RXbuf[B2M_RXtail++];
  #endif
}

void B2M_USART_SendByte(uint8_t Byte){
  #ifndef AVR8
    Serial1.write(Byte);
  #else
    #ifdef UART1_PRESENT
      while (( UCSR1A&(1<<UDRE1)) == 0);
      UDR1 = Byte;
    #else
      while (( UCSR0A&(1<<UDRE0)) == 0);
      UDR0 = Byte;
    #endif
  #endif
}

void B2M_USART_SendBytes(uint8_t *Buf, uint8_t len){
  for(uint8_t i=0; i<len; i++){
    #ifndef AVR8
      Serial1.write(Buf[i]);
    #else
      #ifdef UART1_PRESENT
        while (( UCSR1A&(1<<UDRE1)) == 0);
        UDR1 = Buf[i];
      #else
        while (( UCSR0A&(1<<UDRE0)) == 0);
        UDR0 = Buf[i];
      #endif
    #endif
  }
}

#ifdef AVR8
  #ifdef UART1_PRESENT
    void B2M_USART1_Init(){
        // 2000000 baud
        DDRD |= (1<< PD3);
        DDRD &= ~(1<< PD2);
        UBRR1H = 0;
        UBRR1L = 0;
        UCSR1B = (1<<RXEN1)|(1<<TXEN1)|(1<<RXCIE1);
        UCSR1A = (1<<UDRE1)|(1<<U2X1);
        UCSR1C = (1<<UCSZ11)|(1<<UCSZ10);
    }
    
    ISR(USART1_RX_vect){
      cli();
      B2M_RXbuf[B2M_RXhead++] = UDR1;
      sei();
    }
  #else
    void B2M_USART0_Init(){
        // 2000000 baud
        DDRD |= (1<< PD1);
        DDRD &= ~(1<< PD0);
        UBRR0H = 0;
        UBRR0L = 0;
        UCSR0B = (1<<RXEN0)|(1<<TXEN0)|(1<<RXCIE0);
        UCSR0A = (1<<UDRE0)|(1<<U2X0);
        UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);
    }
    
    ISR(USART0_RX_vect){
      cli();
      B2M_RXbuf[B2M_RXhead++] = UDR0;
      sei();
    }
  #endif
#endif // AVR8



void setup() {
  Arduino_USART_Init();
  B2M_USART_Init();
  FETtecOneWire_Init();
}

void loop() {
  #define MOTOR_COUNT 1
  
  static uint16_t completeTelemetry[MOTOR_COUNT][5] = {0};
  static uint16_t motorValues[MOTOR_COUNT] = {1000};
  static uint8_t TLM_request = 0;

  uint16_t requestedTelemetry[MOTOR_COUNT] = {0};
  
  uint16_t before = micros();
  int8_t TelemetryAvailable = FETtecOneWire_ESCsSetValues(motorValues, requestedTelemetry, MOTOR_COUNT, TLM_request);
  uint16_t onewireProcessTime = (uint16_t)((uint16_t)micros()-(uint16_t)before);
  if(TelemetryAvailable != -1){
    for(uint8_t i = 0; i<MOTOR_COUNT; i++){
      completeTelemetry[i][TelemetryAvailable] = requestedTelemetry[i];
    }
  }
  if(++TLM_request == 5) TLM_request = 0;


  
  #ifdef UART1_PRESENT
  static uint16_t lastPrintTime = 0;
  if((uint16_t)((uint16_t)micros()-(uint16_t)lastPrintTime) >= 50000){
    lastPrintTime = micros();
    Serial.print("onewire process time (us): ");
    Serial.println(onewireProcessTime);
    for(uint8_t i = 0; i<MOTOR_COUNT; i++){
      Serial.print("esc: ");
      Serial.print(i+1);
      Serial.print(" Temperature: ");
      Serial.print(completeTelemetry[i][0]);
      Serial.print(", Voltage: ");
      Serial.print(completeTelemetry[i][1]);
      Serial.print(", Current: ");
      Serial.print(completeTelemetry[i][2]);
      Serial.print(", E-rpm: ");
      Serial.print(completeTelemetry[i][3]);
      Serial.print(", consumption: ");
      Serial.print(completeTelemetry[i][4]);
      Serial.println(" ");
    }

    Serial.println("\n\r\n\r\n\r\n\r\n\r\n\r");
  }
  #endif
  

  // 500Hz loop
  static uint16_t lastLoopTime = 0;
  while((uint16_t)((uint16_t)micros()-(uint16_t)lastLoopTime) < 2000);
  lastLoopTime = micros();
}
